<%-- $Id: index.jsp 7605 2019-02-26 10:10:15Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@since 3.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.DataFileType"
	import="net.sf.basedb.core.ItemSubtypeFileType"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.MultiPermissions"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.ItemAlreadyExistsException"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.util.RemovableUtil"
	import="net.sf.basedb.util.formatter.NameableFormatter"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.clients.web.plugins.ItemQueryLoader"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.edit.OnSaveAction"
	import="net.sf.basedb.clients.web.extensions.edit.OnSaveRenderer"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnExportRenderer"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnAction"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnUtil"
	import="java.util.Enumeration"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.List"
	import="java.util.ArrayList"
	import="java.util.Collections"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%!
	private static final ItemContext defaultContext = Base.createDefaultContext("name", "name,itemType,description");
	private static final Item itemType = Item.ITEMSUBTYPE;

	private static void registerExportUtils(ItemContext cc)
	{
		// Register formatters
		cc.setObject("export.formatter.&fileTypes(dataFileType.name)", new NameableFormatter());
		
		// Register dataloaders
		String subtypeParameter = "subtype";
		// File types
		final ItemQuery<DataFileType> fileTypesQuery = DataFileType.getQuery();
		fileTypesQuery.include(cc.getInclude());
		fileTypesQuery.join(Hql.innerJoin("itemSubtypes", "ist"));
		fileTypesQuery.join(Hql.innerJoin("ist", "itemSubtype", "ist2"));
		fileTypesQuery.restrict(Restrictions.eq(Hql.alias("ist2"), Expressions.parameter("subtype")));
		fileTypesQuery.order(Orders.asc(Hql.property("name"))); 
		cc.setObject("export.dataloader.&fileTypes(dataFileType.name)", new ItemQueryLoader<ItemSubtype>(fileTypesQuery, subtypeParameter));
	}

%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String cmd = request.getParameter("cmd");
final String root = request.getContextPath()+"/";
final String mode = request.getParameter("mode");
final String callback = request.getParameter("callback");
final String itemId = request.getParameter("item_id");
final String listPage = "list_subtypes.jsp?ID="+ID
	+(mode == null ? "" : "&mode="+mode)
	+(callback == null ? "" : "&callback="+callback)
	+(itemId == null ? "" : "&item_id="+itemId);
final String viewPage = "view_subtype.jsp?ID="+ID;
final String editPage = "edit_subtype.jsp?ID="+ID;

String forward = null;
String redirect = null;
String message = null;
DbControl dc = null;

try
{
	if (cmd == null || "List".equals(cmd))
	{
		// Display the list page without updatinging the current context
		Base.getAndSetCurrentContext(sc, itemType, null, defaultContext, true);
		redirect = listPage;
	}
	else if ("UpdateContext".equals(cmd))
	{
		// Display the list page after updating the current context from the request parameters
		Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = listPage;
	}
	else if ("LoadContext".equals(cmd))
	{
		// Display the list page after loading a saved context
		int contextId = Values.getInt(request.getParameter("context"));
		Base.loadContext(sc, contextId, defaultContext);
		redirect = listPage;
	}

	else if ("ViewItem".equals(cmd))
	{
		// Display the view page for a single item 
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		forward = viewPage;
	}
	else if ("EditItem".equals(cmd))
	{
		// Display the edit page for a single item (should be opened in a popup)
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = editPage;
	}
	else if ("NewItem".equals(cmd))
	{
		// Display the edit page for a new item (should be opened in a popup)
		if (!sc.hasPermission(Permission.CREATE, itemType))
		{
			throw new PermissionDeniedException(Permission.CREATE, itemType.toString());
		}
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		cc.setId(0);
		redirect = editPage;
	}
	else if ("UpdateItem".equals(cmd))
	{
		// Update the properties on an item (will close the popup)
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, defaultContext);
		final int maxRecent = Base.getMaxRecent(sc);
		dc = sc.newDbControl();
		ItemSubtype subtype = cc.getObject("item");
		if (subtype == null)
		{
			Item mainItemType = Item.valueOf(request.getParameter("itemType"));
			subtype = ItemSubtype.getNew(dc, mainItemType);
			message = "Item subtype created";
			dc.saveItem(subtype);
		}
		else
		{
			dc.reattachItem(subtype, false);
			message = "Item subtype updated";
		}
		JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), subtype);
		ExtensionsInvoker<OnSaveAction> invoker = EditUtil.useOnSaveExtensions(jspContext);
		try
		{
			subtype.setName(Values.getStringOrNull(request.getParameter("name")));
			subtype.setDescription(Values.getStringOrNull(request.getParameter("description")));
			subtype.setPushAnnotations(Values.getBoolean(request.getParameter("pushAnnotations")));
			
			for (Item related : ItemSubtype.getRelatedItems(subtype.getMainItemType()))
			{
				int relatedId = Values.getInt(request.getParameter("related." + related.name()), -1);
				if (relatedId >= 0) // < 0 = denied or unchanged
				{
					if (relatedId == 0)
					{
						subtype.removeRelatedSubtype(related);
					}
					else
					{
						subtype.setRelatedSubtype(ItemSubtype.getById(dc, relatedId));
					}
				}
			}

			// Data file types
			String[] modifiedFileTypes = Values.getString(request.getParameter("+DATAFILETYPE")).split(",");
			for (int i = 0; i < modifiedFileTypes.length; ++i)
			{
				int ftId = Values.getInt(modifiedFileTypes[i], -1);
				if (ftId != -1) 
				{
					DataFileType dft = DataFileType.getById(dc, ftId);
					int options = Values.getInt(request.getParameter("DATAFILETYPE."+ftId));
					ItemSubtypeFileType sft = subtype.getAssociatedDataFileType(dft, true);
					sft.setRequired((options & 1) > 0);
					sft.setAllowMultiple((options & 2) > 0);
				}
			}
			String[] removedFileTypes = Values.getString(request.getParameter("-DATAFILETYPE")).split(",");
			for (int i = 0; i < removedFileTypes.length; ++i)
			{
				int ftId = Values.getInt(removedFileTypes[i], -1);
				if (ftId != -1) 
				{
					DataFileType dft = DataFileType.getById(dc, ftId);
					subtype.removeAssociatedDataFileType(dft);
				}
			}

			cc.removeAllTemporaryFilters();
			
			// OnSave extensions
			invoker.render(OnSaveRenderer.ON_SAVE);
			dc.commit();
			invoker.render(OnSaveRenderer.ON_COMMIT);
		}
		catch (Exception ex)
		{
			invoker.render(OnSaveRenderer.onRollback(ex));
			throw ex;
		}
		finally
		{
			cc.removeObject("item");
		}
	}
	else if ("DeleteItem".equals(cmd))
	{
		// Delete a single item and then return to the view page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		RemovableUtil.setRemoved(dc, itemType, Collections.singleton(cc.getId()), true);
		dc.commit();
		redirect = viewPage;
	}
	else if ("DeleteItems".equals(cmd))
	{
		// Delete all selected items on the list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		int numTotal = cc.getSelected().size();
		int numRemoved = RemovableUtil.setRemoved(dc, itemType, cc.getSelected(), true);
		dc.commit();
		if (numTotal != numRemoved)
		{
			message = (numRemoved == 0 ? "No" : "Only "+numRemoved+" of "+numTotal) + " items could be deleted, because you have no DELETE permission";
		}
		redirect = listPage;
	}
	else if ("RestoreItem".equals(cmd))
	{
		// Restore a single item and then return to the view page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		RemovableUtil.setRemoved(dc, itemType, Collections.singleton(cc.getId()), false);
		dc.commit();
		redirect = viewPage;
	}
	else if ("RestoreItems".equals(cmd))
	{
		// Restore all selected items on the list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		int numTotal = cc.getSelected().size();
		int numRemoved = RemovableUtil.setRemoved(dc, itemType, cc.getSelected(), false);
		dc.commit();
		if (numTotal != numRemoved)
		{
			message = (numRemoved == 0 ? "No" : "Only "+numRemoved+" of "+numTotal) + " items could be restored, because you have no WRITE permission";
		}
		redirect = listPage;
	}
	else if ("ExportItems".equals(cmd))
	{
		// Run an export plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		final ItemQuery<ItemSubtype> query = ItemSubtype.getQuery(null);
		dc = sc.newDbControl();
		cc.configureQuery(dc, query, true);
		cc.setQuery(query);
		registerExportUtils(cc);
		JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.list(itemType), null);
		ExtensionsInvoker<ListColumnAction<ItemSubtype,?>> listInvoker = ListColumnUtil.useExtensions(jspContext);
		listInvoker.render(new ListColumnExportRenderer<ItemSubtype>(cc));
		dc.close();
		redirect = "../../common/export/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=LIST&title=Export+item+subtypes";
	}
	else if ("ExportItem".equals(cmd))
	{
		// Run an export plugin in single-item context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		registerExportUtils(cc);
		redirect = "../../common/export/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=ITEM&title=Export+item+subtype";
	}
	else if ("ImportItems".equals(cmd))
	{
		// Run an import plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		final ItemQuery<ItemSubtype> query = ItemSubtype.getQuery(null);
		dc = sc.newDbControl();
		cc.configureQuery(dc, query, true);
		dc.close();
		cc.setQuery(query);
		redirect = "../../common/import/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=LIST&title=Import+item+subtypes";
	}
	else if ("ImportItem".equals(cmd))
	{
		// Run an import plugin in single-item context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = "../../common/import/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=ITEM&title=Import+item+subtype";
	}
	else if ("RunListPlugin".equals(cmd))
	{
		// Run another plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		final ItemQuery<ItemSubtype> query = ItemSubtype.getQuery(null);
		dc = sc.newDbControl();
		cc.configureQuery(dc, query, true);
		dc.close();
		cc.setQuery(query);
		redirect = "../../common/plugin/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=LIST&main_type=OTHER&title=Run+plugin";
	}
	else if ("RunPlugin".equals(cmd))
	{
		// Run another plugin in single-item context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = "../../common/plugin/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=ITEM&main_type=OTHER&title=Run+plugin";
	}
	else
	{
		throw new WebException("popup", "Invalid command", "The command {1} is not recognised as a valid command.", cmd);
	}
}
finally
{
	if (dc != null) dc.close();
}

if (forward != null)
{
	sc.setSessionSetting("alert-message", message);
	pageContext.forward(forward);
}
else if (redirect != null)
{
	sc.setSessionSetting("alert-message", message);
	response.sendRedirect(redirect);
}
else if (message == null)
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&wait=0");
}
else
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&message="+HTML.urlEncode(message));
}
%>

