<%-- $Id: edit_agent.jsp 7604 2019-02-25 12:19:50Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.JobAgent"
	import="net.sf.basedb.core.JobAgentSettings"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.tabcontrol.TabAction"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.util.json.NameableConverter"
	import="org.json.simple.JSONArray"
	import="org.json.simple.JSONObject"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final Item itemType = Item.JOBAGENT;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	String title = null;
	JobAgent agent = null;
	Integer port = null;

	final int selectedPluginId = Values.getInt(request.getParameter("plugin_id"));

	// Query to retrieve installed plugins
	ItemQuery<PluginDefinition> pluginQuery = PluginDefinition.getQuery();
	pluginQuery.include(Include.ALL);
	pluginQuery.order(Orders.asc(Hql.property("name")));

	if (itemId == 0)
	{
		title = "Create job agent";
		cc.removeObject("item");
		port = Values.getInteger(cc.getPropertyValue("port"), JobAgent.DEFAULT_PORT);
	}
	else
	{
		agent = JobAgent.getById(dc, itemId);
		agent.checkPermission(Permission.WRITE);
		cc.setObject("item", agent);
		title = "Edit job agent -- " + HTML.encodeTags(agent.getName());
		port = agent.getPort();
	}
	
	// Load plugin definitions as JSON objects
	JSONArray jsonSettings = new JSONArray();
	JSONObject jsonPlugins = new JSONObject();
	jsonPlugins.put("itemType", "PLUGINDEFINITION");
	if (pluginQuery != null)
	{
		JSONArray jsonLinked = new JSONArray();
		NameableConverter<PluginDefinition> converter = new NameableConverter<>();
		for (PluginDefinition plugin : pluginQuery.list(dc))
		{
			// Default settings
			boolean defaultTrusted = plugin.isTrusted();
			Long defaultMaxMemory = plugin.getMaxMemory();

			// Specific settings
			JobAgentSettings settings = agent == null ? null : agent.getSettings(plugin, false);
			Boolean trusted = null;
			Long maxMemory = null;
			int priorityBoost = 0;
			if (settings != null)
			{
				trusted = settings.isTrusted();
				maxMemory = settings.getMaxMemory();
				priorityBoost = settings.getPriorityBoost();
				jsonLinked.add(converter.convert(plugin));
				
			}
			JSONObject jsonSetting = new JSONObject();
			jsonSetting.put("pluginId", plugin.getId());
			jsonSetting.put("defaultTrusted", defaultTrusted ? 1 : 0);
			if (trusted != null) 
			{
				jsonSetting.put("trusted", trusted ? 1 : 0);
			}
			else
			{
				jsonSetting.put("trusted", "");
			}
			if (defaultMaxMemory != null)
			{
				jsonSetting.put("defaultMaxMemory", Values.formatBytes(defaultMaxMemory));
			}
			if (maxMemory != null)
			{
				jsonSetting.put("maxMemory", Values.formatBytes(maxMemory));
			}
			jsonSetting.put("priorityBoost", priorityBoost);
			jsonSettings.add(jsonSetting);
		}	
		jsonPlugins.put("items", jsonLinked);
	}

	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), agent);
	ExtensionsInvoker<TabAction> invoker = EditUtil.useEditExtensions(jspContext);
	%>
	<base:page type="popup" title="<%=title%>" id="edit-page">
	<base:head scripts="tabcontrol-2.js,linkitems-2.js,~agents.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%> <base:help tabcontrol="settings" /></h1>
		<form action="index.jsp?ID=<%=ID%>" method="post" name="agent">
		<input type="hidden" name="cmd" value="UpdateItem">

		<div id="page-data" class="datacontainer"
			data-plugin-settings="<%=HTML.encodeTags(jsonSettings.toJSONString()) %>"
			data-select-plugin="<%=selectedPluginId%>"
			></div>

		<t:tabcontrol id="settings" 
			subclass="content dialogtabcontrol"
			position="bottom" remember="<%=agent != null%>" 
			active="<%=selectedPluginId == 0 ? null : "plugins" %>"
			extensions="<%=invoker%>">
		<t:tab id="info" title="Job agent" helpid="jobagent.edit">
			<table class="fullform input100 smaller">
			<tr>
				<th>Name</th>
				<td><input class="text required auto-init" data-auto-init="<%=agent == null ? "focus-select" : "focus" %>"
					type="text" name="name" 
					value="<%=HTML.encodeTags(agent == null ? Values.getString(cc.getPropertyValue("name"), "New job agent") : agent.getName())%>" 
					maxlength="<%=JobAgent.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>External Id</th>
				<td><input class="text required" type="text" name="external_id" 
					value="<%=HTML.encodeTags(agent == null ? cc.getPropertyValue("externalId") : agent.getExternalId())%>" 
					maxlength="<%=JobAgent.MAX_EXTERNAL_ID_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>Server</th>
				<td><input class="text" type="text" name="server" 
					value="<%=HTML.encodeTags(agent == null ? cc.getPropertyValue("server") : agent.getServer())%>" 
					maxlength="<%=JobAgent.MAX_SERVER_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>Port</th>
				<td><input class="text" type="text" name="port" id="port"
					value="<%=port == null ?  "" : port.toString() %>" 
					maxlength="10" style="width: 15em;">
					<span class="link" id="useDefaultPort" 
						data-port="<%=JobAgent.DEFAULT_PORT%>"
						>Use default (<%=JobAgent.DEFAULT_PORT%>)</span>
					</td>
				<td></td>
			</tr>
			<tr class="dynamic">
				<th>Description</th>
				<td>
					<textarea class="text" rows="6" name="description" id="description"
						><%=HTML.encodeTags(agent == null ? cc.getPropertyValue("description") : agent.getDescription())%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="description" title="Description" />
				</td>
			</tr>
			</table>
		</t:tab>
		
		<t:tab id="plugins" title="Plugins" helpid="jobagent.plugins">
			<table class="fullform input100 smaller">
			<tr class="big">
				<th>Installed plugins</th>
				<td>
					<div class="selectionlist">
						<table>
						<tr>
							<td>
								<select name="plugins" id="plugins" 
									class="auto-init" data-auto-init="link-container"
									data-initial-items="[<%=HTML.encodeTags(jsonPlugins.toJSONString()) %>]"
									size="12" multiple>
								</select>
							</td>
							<td style="vertical-align: top;">
								<base:buttongroup vertical="true">
									<base:button 
										id="btnAddPlugins"
										subclass="leftaligned auto-init"
										data-auto-init="add-link"
										data-list-id="plugins"
										data-item-type="PLUGINDEFINITION"
										title="Add&nbsp;plugins&hellip;"  
										tooltip="Add plugins to this job agent"
									/>
									<base:button 
										id="btnRemovePlugins"
										subclass="leftaligned auto-init"
										data-auto-init="remove-link"
										data-list-id="plugins"
										title="Remove" 
										tooltip="Remove the selected plugins"
									/>
								</base:buttongroup>
							</td>
						</tr>
						</table>
					</div>
				</td>
			</tr>
			<tbody class="sectionheader">
				<tr>
					<th colspan="2">Settings for the selected plug-in</th>
				</tr>
			</tbody>
			<tbody id="pluginsettings" style="display: none;">
				<tr>
					<th class="subprompt">Max memory</th>
					<td>
						<input type="text" name="maxMemory" id="maxMemory" class="text" style="width: 15em;"
							maxlength="16">(Use KB, MB or GB to specify memory)
					</td>
				</tr>
				<tr>
					<th class="subprompt"></th>
					<td>
						<input type="checkbox" name="useDefaultMaxMemory" id="useDefaultMaxMemory">
						<label for="useDefaultMaxMemory">Use default</label> (<span id="defaultMaxMemory"></span>)
					</td>
				</tr>
				<tr>
					<th class="subprompt">Trusted</th>
					<td>
						<input type="radio" name="trusted" id="trustedDefault" value="" 
							><label for="trustedDefault">default (<span id="defaultTrusted"></span>)</label>
						<input type="radio" name="trusted" id="trustedYes" value="1"
							><label for="trustedYes">yes</label>
						<input type="radio" name="trusted" id="trustedNo" value="0"
							><label for="trustedNo">no</label>
					</td>
				</tr>
				<tr>
					<th class="subprompt">Priority boost</th>
					<td>
						<input type="text" name="priorityBoost" id="priorityBoost" class="text" style="width: 15em;"
							maxlength="10">
					</td>
				</tr>
			</tbody>
			<tr class="dynamic">
				<th></th>
				<td></td>
			</tr>
			</table>
		</t:tab>
		</t:tabcontrol>
		</form>
		
		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
