<%-- $Id: view_user.jsp 7605 2019-02-26 10:10:15Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Group"
	import="net.sf.basedb.core.Quota"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.Role"
	import="net.sf.basedb.core.Directory"
	import="net.sf.basedb.core.ExtendedProperties"
	import="net.sf.basedb.core.ExtendedProperty"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.PermissionUtil"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.EmailUtil"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction" 
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.Map"
	import="java.util.Date"
	import="java.util.List"	
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="p" uri="/WEB-INF/path.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.USER;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.ITEM);
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);

	String title = null;
	User user = User.getById(dc, itemId);

	final boolean createPermission = sc.hasPermission(Permission.CREATE, itemType);
	final boolean writePermission = user.hasPermission(Permission.WRITE);
	final boolean deletePermission = user.hasPermission(Permission.DELETE);
	final boolean isRemoved = user.isRemoved();
	final boolean isUsed = isRemoved && user.isUsed();
	final boolean deletePermanentlyPermission = deletePermission && !isUsed;
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, guiContext, user);
	ExtensionsInvoker<ButtonAction> invoker = ToolbarUtil.useExtensions(jspContext);
	%>
	<base:page title="<%=title%>" id="view-page">
	<base:head scripts="table.js,tabcontrol-2.js,~users.js" styles="toolbar.css,table.css,headertabcontrol.css,path.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<p:path><p:pathelement 
			title="Users" href="<%="index.jsp?ID="+ID%>" 
			/><p:pathelement title="<%=HTML.encodeTags(user.getName())%>" 
			/></p:path>
			
		<div id="page-data" data-item-id="<%=itemId%>"></div>
		
		<t:tabcontrol 
			id="main" 
			subclass="content mastertabcontrol" 
			active="properties">
		<t:tab id="properties" title="Properties">
			<div>
			<table class="fullform">
			<tr>
				<th class="itemstatus">
					<base:icon 
						id="btnDeletePermanently"
						image="deleted.png"
						tooltip="This item has been flagged for deletion. Click to delete it now."
						enabled="<%=deletePermanentlyPermission %>"
						visible="<%=isRemoved%>" 
					/>
					<base:icon 
						id="btnUsingItems"
						image="used.png" 
						tooltip="This item is used by other items and can't be permanently deleted. Show the items that are using this one"
						visible="<%=isRemoved && isUsed%>" />
				</th>
				<td style="padding: 0px;">
					<tbl:toolbar subclass="bottomborder bg-filled-50">
						<tbl:button 
							id="btnEdit"
							disabled="<%=!writePermission%>" 
							image="edit.png" 
							title="Edit&hellip;" 
							tooltip="<%=writePermission ? "Edit this user" : "You do not have permission to edit this user"%>" 
						/>
						<tbl:button 
							id="btnDelete"
							disabled="<%=!deletePermission%>" 
							image="delete.png" 
							title="Delete"
							visible="<%=!user.isRemoved()%>"
							tooltip="<%=deletePermission ? "Delete this user" : "You do not have permission to delete this user"%>" 
						/>
						<tbl:button 
							id="btnRestore"
							disabled="<%=!writePermission%>" 
							image="restore.png" 
							title="Restore"
							visible="<%=user.isRemoved()%>"
							tooltip="<%=writePermission ? "Restore this user" : "You do not have permission to restore this user"%>" 
						/>
						<tbl:button 
							id="btnClone"
							disabled="<%=!createPermission %>"
							image="copy.png" 
							title="Clone&hellip;" 
							tooltip="Create a new user using settings from this user as a template" 
						/>
						<tbl:button 
							id="btnImport"
							image="import.png" 
							data-plugin-type="IMPORT"  
							title="Import&hellip;" 
							tooltip="Import data" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
						/>
						<tbl:button 
							id="btnExport"
							image="export.png"
							data-plugin-type="EXPORT" 
							title="Export&hellip;" 
							tooltip="Export data" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
						/>
						<tbl:button 
							id="btnRunPlugin"
							image="runplugin.png"  
							data-plugin-type="OTHER" 
							title="Run plugin&hellip;" 
							tooltip="Run a plugin" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
						/>
						<ext:render extensions="<%=invoker%>" context="<%=jspContext%>" 
							wrapper="<%=new PrefixSuffixRenderer<ButtonAction>(jspContext, "<td>", "</td>") %>"/>
						<tbl:button
							image="help.png"
							subclass="auto-init"
							data-auto-init="help"
							data-help-id="user.view.properties"
							title="Help&hellip;"
							tooltip="Get help about this page"
						/>
					</tbl:toolbar>
				</td>
			</tr>
			</table>
			<table style="width: 100%; height: 22em;" class="bottomborder">
			<tr>
				<td style="width: 50%; height: 100%;">
					<table class="fullform" >
					<tr>
						<th>Name</th>
						<td><%=HTML.encodeTags(user.getName())%></td>
					</tr>
					<tr>
						<th>Registered</th>
						<td><%=dateFormatter.format(user.getEntryDate())%></td>
					</tr>
				<tr>
					<th>System ID</th>
					<td><%=HTML.encodeTags(user.getSystemId())%></td>
				</tr>
				<tr>
					<th>Login</th>
					<td><%=HTML.encodeTags(user.getLogin())%></td>
				</tr>
				<tr>
					<th>External ID</th>
					<td><%=HTML.encodeTags(user.getExternalId())%></td>
				</tr>
				<tr>
					<th>Home directory</th>
					<td><base:propertyvalue item="<%=user%>" property="homeDirectory" /></td>
				</tr>
				<tr>
					<th>Quota</th>
					<td><base:propertyvalue item="<%=user%>" property="quota" /></td>
				</tr>
				<tr >
					<th>Quota group</th>
					<td><base:propertyvalue item="<%=user%>" property="quotaGroup" /></td>
				</tr>
				<tr>
					<th>Expiration date</th>
					<td><%=user.getExpirationDate() == null ? "<i>- never -</i>" : 
						dateFormatter.format(user.getExpirationDate())%></td>
				</tr>
				<tr>
					<th>Multi-user account</th>
					<td><%=user.isMultiuserAccount() ? "yes" : "no"%></td>
				</tr>
				<tr>
					<th>2-factor login</th>
					<td><%=user.getUseDeviceVerification() ? "yes" : "no"%>
					<%
					if (!EmailUtil.isEnabled() && user.getUseDeviceVerification()) 
					{
						%>
						(You need to configure an email server in <i>base.config</i> before you can use this feature.)
						<%
					}
					%>
					</td>
				</tr>
				<tr>
					<th>Disabled</th>
					<td><%=user.isDisabled() ? "yes" : "no"%></td>
				</tr>
				<tr>
					<th>Permissions</th>
					<td><%=PermissionUtil.getFullPermissionNames(user)%></td>
				</tr>
				<tr>
					<th>Description</th>
					<td><%=HTML.niceFormat(user.getDescription())%></td>
				</tr>
				</table>
			</td>
			<td style="width: 50%; height: 100%;" class="leftborder">
				<table class="fullform">
				<tbody class="sectionheader">
					<tr>
						<th colspan="2">Contact information</th>
					</tr>
				</tbody>
				<tr>
					<th>Email</th>
					<td><%=HTML.niceFormat(user.getEmail())%></td>
				</tr>
				<tr>
					<th>Organisation</th>
					<td><%=HTML.encodeTags(user.getOrganisation())%></td>
				</tr>
				<tr>
					<th>Address</th>
					<td><%=HTML.niceFormat(user.getAddress())%></td>
				</tr>
				<tr>
					<th>Phone</th>
					<td><%=HTML.encodeTags(user.getPhone())%></td>
				</tr>
				<tr>
					<th>Fax</th>
					<td><%=HTML.encodeTags(user.getFax())%></td>
				</tr>
				<tr class="dynamic">
					<th></th>
					<td></td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
		</div>
		
			<%
			List<ExtendedProperty> extendedProperties = ExtendedProperties.getProperties("UserData");
			if (extendedProperties != null)
			{
				%>
				<base:section 
					id="extraInfo" 
					title="<%="Additional information (" + extendedProperties.size() + ")"%>"
					>
					<table class="fullform fullborder">	
						<%
						boolean needsTr = true;
						for (ExtendedProperty ep : extendedProperties)
						{
							String name = ep.getName();
							Formatter<Object> f = FormatterFactory.getExtendedPropertyFormatter(sc, ep);
							String value = f.format(user.getExtended(name));
							%>
								<%=needsTr ? "<tr><th>" : "<th class=\"leftborder\">" %>
								<%=HTML.encodeTags(ep.getTitle())%></th>
								<td><%=value%></td>
								<%=needsTr ? "" : "</tr>"%>
							<%
							needsTr = !needsTr;
						}
						%>
						<%=needsTr ? "" : "<th class=\"leftborder\"></th><td></td></tr>"%>
					</table>
				</base:section>
				<%
			}

			ItemQuery<Group> groupQuery = user.getGroups();
			groupQuery.include(Include.ALL);
			groupQuery.order(Orders.asc(Hql.property("name")));
			ItemResultList<Group> groups = groupQuery.list(dc);
			
			ItemQuery<Role> roleQuery = user.getRoles();
			roleQuery.include(Include.ALL);
			roleQuery.order(Orders.asc(Hql.property("name")));
			ItemResultList<Role> roles = roleQuery.list(dc);
			int numMembership = groups.size() + roles.size();
			%>
			<base:section 
				id="membership" 
				title="<%="Membership (" + numMembership + ")"%>"
				context="<%=cc%>"
				>
				<%
				if (numMembership == 0)
				{
					%>
					<div class="messagecontainer note">
					This user isn't member of any group or role (or, you don't have permission to view them).
					</div>
					<%
				}
				else
				{
					%>
					<tbl:table
						id="tbl.membership"
						columns="all"
						>
					<tbl:columndef 
						id="name"
						title="Name"
					/>
					<tbl:columndef 
						id="type"
						title="Type"
					/>
					<tbl:columndef 
						id="description"
						title="Description"
					/>
					<tbl:data>
						<tbl:headers>
							<tbl:headerrow>
								<tbl:columnheaders />
							</tbl:headerrow>
						</tbl:headers>
						<tbl:rows>
						<%
						for (Group group : groups)
						{
							%>
							<tbl:row>
								<tbl:cell column="name"><base:icon 
										image="deleted.png" 
										tooltip="This item has been scheduled for deletion" 
										visible="<%=group.isRemoved()%>"
									/><%=Base.getLinkedName(ID, group, false, true)%></tbl:cell>
								<tbl:cell column="type">Group</tbl:cell>
								<tbl:cell column="description"><%=HTML.encodeTags(group.getDescription())%></tbl:cell>
							</tbl:row>
							<%
						}
						for (Role role : roles)
						{
							%>
							<tbl:row>
								<tbl:cell column="name"><base:icon 
										image="deleted.png" 
										tooltip="This item has been scheduled for deletion" 
										visible="<%=role.isRemoved()%>"
									/><%=Base.getLinkedName(ID, role, false, true)%></tbl:cell>
								<tbl:cell column="type">Role</tbl:cell>
								<tbl:cell column="description"><%=HTML.encodeTags(role.getDescription())%></tbl:cell>
							</tbl:row>
							<%
						}
						%>
						</tbl:rows>
					</tbl:data>
					</tbl:table>
					<%
				}
				%>
			</base:section>
			<jsp:include page="../../common/anytoany/list_anytoany.jsp">
				<jsp:param name="ID" value="<%=ID%>" />
				<jsp:param name="item_type" value="<%=itemType.name()%>" />
				<jsp:param name="item_id" value="<%=itemId%>" />
				<jsp:param name="title" value="Other items related to this user" />
			</jsp:include>
		</t:tab>
		</t:tabcontrol>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}

%>
