<%-- $Id: index.jsp 6607 2014-11-19 14:52:20Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.Application"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Config"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Directory"
	import="net.sf.basedb.core.File"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.Group"
	import="net.sf.basedb.core.Quota"
	import="net.sf.basedb.core.QuotaType"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.Location"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.BaseException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.util.zip.FileUnpacker"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="java.util.List"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.nio.charset.Charset"
	import="org.json.simple.JSONObject"
	import="org.json.simple.JSONArray"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%
final Item itemType = Item.FILE;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	String title = null;
	File file = null;
	Directory directory = null;
	boolean readCurrentSubtype = true;
	int currentSubtypeId = 0;
	Boolean compress = Application.autoCompressionEnabled() ? null : false; // null == auto

	if (itemId == 0)
	{
		title = "Upload new file";
		currentSubtypeId = Values.getInt(cc.getPropertyValue("itemSubtype"), 0);
		if (currentSubtypeId == 0)
		{
			currentSubtypeId = Values.getInt(cc.getRecent(Item.ITEMSUBTYPE.name(), 0), 0);
		}
		directory = Directory.getById(dc, Values.getInt(request.getParameter("directory_id"), SystemItems.getId(Directory.ROOT)));
		cc.removeObject("item");
		if (directory.getAutoCompress()) compress = true;
	}
	else
	{
		file = File.getById(dc, itemId);
		file.checkPermission(Permission.WRITE);
		
		directory = file.getDirectory();
		cc.setObject("item", file);
		title = "Upload again -- " + HTML.encodeTags(file.getName());
		try
		{
			ItemSubtype subtype = file.getItemSubtype();
			if (subtype != null) currentSubtypeId = subtype.getId();
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentSubtype = false;
		}
	}
	
	if (file == null || file.getLocation() != Location.PRIMARY)
	{
		// Check quota
		User user = User.getById(dc, sc.getLoggedInUserId());
		QuotaType totalQuotaType = QuotaType.getById(dc, SystemItems.getId(QuotaType.TOTAL));
		QuotaType fileQuotaType = QuotaType.getById(dc, SystemItems.getId(QuotaType.FILE));
		
		if (user.hasExceededQuota(totalQuotaType, Location.PRIMARY) 
			|| user.hasExceededQuota(fileQuotaType, Location.PRIMARY))
		{
			throw new PermissionDeniedException("Quota has been exceeded");
		}
	}
	
	// Query to retrieve file types
	final ItemQuery<ItemSubtype> subtypesQuery = Base.getSubtypesQuery(itemType);
	subtypesQuery.include(Include.ALL);
	
	JSONArray jsonZipFormats = new JSONArray();
	if (file == null)
	{
		// Query to retreive FileUnpacker plugins (used for new uploads only)
		final ItemQuery<PluginDefinition> pluginQuery = PluginDefinition.getQuery(null, "net.sf.basedb.util.zip.FileUnpacker");
		pluginQuery.order(Orders.asc(Hql.property("name")));
		pluginQuery.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
		List<PluginDefinition> plugins = pluginQuery.list(dc);
		
		for (PluginDefinition plugin : plugins)
		{
			FileUnpacker unpacker = plugin.newInstance(FileUnpacker.class, sc, null, null);
			JSONObject jsonZipFormat = new JSONObject();
			jsonZipFormat.put("id", plugin.getId());
			jsonZipFormat.put("name", unpacker.getFormatName());
			JSONArray jsonExtensions = new JSONArray();
			jsonExtensions.addAll(unpacker.getExtensions());
			jsonZipFormat.put("extensions", jsonExtensions);
			jsonZipFormats.add(jsonZipFormat);
		}
	}
	%>
	<base:page type="popup" title="<%=title%>" id="select-file">
	<base:head scripts="~upload.js" styles="progressbar.css" />
	<base:body data-notify-on-error="1">
	<div id="selectForm" class="absolutefull">
		<h1><%=title%> <base:help helpid="file.upload"/></h1>
		
		<div id="page-data" class="datacontainer"
			data-zip-formats="<%=HTML.encodeTags(jsonZipFormats.toJSONString()) %>"
		></div>
		
		<form action="Upload?ID=<%=ID%>&amp;directory_id=<%=directory.getId()%>" 
			method="post" name="file" enctype="multipart/form-data" target="upload">

		<input type="hidden" name="check_md5" value="0">
		<input type="hidden" name="write_protected" value="0">
		<input type="hidden" name="replace_existing" value="0">
		<input type="hidden" name="compressed" value="">
		<input type="hidden" name="zip_format" value="">
		<input type="hidden" name="zip_unpack" value="0">
		<input type="hidden" name="zip_overwrite" value="0">
		<input type="hidden" name="zip_keep" value="1">
		<input type="hidden" name="subtype_id" value="">
		<input type="hidden" name="description" value="">
		<input type="hidden" name="characterSet" value="">

		<div class="content">
			<table class="fullform input100 bottomborder">
			<tr>
				<th>Directory</th>
				<td colspan="2"><%=HTML.encodeTags(directory.getPath().toString())%></td>
			</tr>
			<tr>
				<th>File</th>
				<td>
					<input class="text required" type="file" name="the_file" id="the_file" size="40">
				</td>
				<td></td>
			</tr>
			<tr>
				<th></th>
				<td>
					<input type="checkbox" name="temp_write_protected" id="tempWriteProtected" value="1" 
					<%=(file != null && file.isWriteProtected()) || 
						(file == null && Values.getBoolean(cc.getPropertyValue("writeProtected"))) ? "checked" : ""%>>
					<label for="tempWriteProtected">Write protected</label>
					<%
					if (file == null)
					{
						%>
						<input type="checkbox" name="temp_replace_existing" id="tempReplaceExisting" value="1">
						<label for="tempReplaceExisting">Replace existing</label>
						<%
					}
					else
					{
						%>
						<input type="checkbox" name="temp_check_md5" id="tempCheckMd5">
						<label for="tempCheckMd5">Validate MD5</label>
						<%
					}
					%>
				</td>
				<td></td>
			</tr>
			<tr>
				<th>Store compressed</th>
				<td>
					<select name="temp_compressed">
						<%
						if (Application.autoCompressionEnabled())
						{
							%>
							<option value="" <%=compress == null ? "selected" : "" %>>- auto -
							<%
						}
						%>
						<option value="1" <%=Boolean.TRUE.equals(compress) ? "selected" : ""%>>yes
						<option value="0" <%=Boolean.FALSE.equals(compress) ? "selected" : ""%>>no
					</select>
				</td>
				<td></td>
			</tr>
			<tr >
				<th>Type</th>
				<td>
					<select name="temp_subtype_id" 
						<%=!readCurrentSubtype ? "disabled readonly class=\"disabled selectionlist\"" : "class=\"selectionlist\""%>>
					<%
					if (!readCurrentSubtype)
					{
						%>
						<option value="-1">- denied -
						<%
					}
					else
					{
						%>
						<option value="0">- none -
						<%
						for (ItemSubtype subtype : subtypesQuery.list(dc))
						{
							int id = subtype.getId();
							boolean current = id == currentSubtypeId;
							%>
							<option 
								value="<%=current && file != null ? -id : id%>" 
								<%=current ? "selected" : ""%>
								title="<%=HTML.encodeTags(subtype.getDescription())%>"
								><%=HTML.encodeTags(subtype.getName())%>
							<%
						}
					}
					%>
					</select>
				</td>
				<td></td>
			</tr>
			<tr>
				<th>Character set</th>
				<td>
					<select name="temp_charset">
					<option value="">- n/a -
					<%
					String currentCharset = file == null ? null : file.getCharacterSet();
					int numCommon = Config.getCommonCharsets().size();
					int i = 0;
					boolean hasSelected = false;
					for (String charset : Config.getAllCharsets())
					{
						boolean selected = false;
						if (!hasSelected)
						{
							selected = charset.equalsIgnoreCase(currentCharset);
							hasSelected |= selected;
						}
						%>
						<option value="<%=charset%>" <%=selected ? "selected" : ""%> 
							<%=i==numCommon ? "style=\"border-top: 1px solid #666666;\"" : "" %>><%=charset%>
						<%
						i++;
					}
					if (!hasSelected && currentCharset != null)
					{
						%>
						<option value="<%=currentCharset%>" selected><%=currentCharset%>
						<%
					}
					%>
					</select>
				</td>
				<td></td>
			</tr>
			<%
			if (file == null)
			{
				%>
				<tr class="big">
					<th>Compressed file</th>
					<td>
						<select name="temp_zip_format" disabled>
						</select><br>
					
						<input type="checkbox" name="temp_zip_unpack" id="tempZipUnpack" disabled
							><label for="tempZipUnpack">Unpack file</label><br>
						<input type="checkbox" name="temp_zip_overwrite" id="tempZipOverwrite" disabled
							><label for="tempZipOverwrite">Overwrite existing files</label><br>
						<input type="checkbox" name="temp_zip_keep" id="tempZipKeep" disabled checked
							><label for="tempZipKeep">Keep the compressed file</label><br>
					</td>
					<td></td>
				</tr>
				<%
			}
			%>
			<tr class="dynamic">
				<th>Description</th>
				<td>
					<textarea class="text" rows="4" name="temp_description" id="temp_description"
						><%=HTML.encodeTags(file == null ? cc.getPropertyValue("description") : file.getDescription())%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="temp_description" title="Description" />
				</td>
			</tr>
			</table>
		</div>
		</form>

		<div class="legend" style="height: 1.75rem;">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnUpload" image="upload.png" title="Upload" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</div>
	
	<div id="progressForm" class="absolutefull" style="display:none;">
		<h1 class="docked" id="fileName">Preparing file upload...</h1>
		<div class="content">
			<table class="fullcc bg-filled-50 bottomborder"><tr><td>
				<table class="progressbar" style="width: 100%;">
				<tr>
					<td style="width: 2em;"></td>
					<td style="">
						<table class="bar" style="width: 100%;">
						<tr>
							<td style="width: 0%;" class="percentDone" id="percentDone">&nbsp;</td>
							<td style="width: 100%;" class="percentRemain" id="percentRemain">&nbsp;</td>
						</tr>
						</table>
					</td>
					<td style="width: 3em;" class="percentText" id="percentText">0%</td>
					<td style="width: 2em;"></td>
				</tr>
				<tr>
					<td></td>
					<td colspan="2" style="text-align: left;">
						<b>Transferred:</b> <span id="transferredBytes">0 bytes</span> of <span id="totalBytes">unknown</span><br>
						<b>Time:</b> <span id="transferTime">0s</span> <span id="transferRate"></span><br>
						<b>Remain:</b> <span id="remainTime">unknown</span><br>
						<span id="message"></span>
					</td>
					<td></td>
				</tr>
				</table>
			</td>
			</tr>
			</table>
		</div>
		
		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnAbort" image="cancel.png" title="Abort..." />
		</base:buttongroup>
	</div>
	
	<div id="uploadForm" class="absolutefull" 
		style="display: none;"><iframe 
		src="../wait.jsp"
		name="upload" class="absolutefull" 
		style="width:100%; height: 100%;"></iframe></div>
	
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>



