<%-- $Id: submit_user.jsp 7605 2019-02-26 10:10:15Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	This page will save user information and preferences.

	@param cmd The command to execute
		- SaveSettings: Save the settings from the settings.jsp page.
		- SavePreferences: Save preferences from the preferences.jsp page.

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.ExtendedProperty"
	import="net.sf.basedb.core.ExtendedProperties"
	import="net.sf.basedb.util.EmailUtil"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.clients.web.formatter.FormatterSettings"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.edit.OnSaveAction"
	import="net.sf.basedb.clients.web.extensions.edit.OnSaveRenderer"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="java.util.Arrays"
	import="java.util.Collections"
	import="java.util.List"
	import="java.util.Map"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%
SessionControl sc = Base.getExistingSessionControl(pageContext, true);
String ID = sc.getId();
String cmd = request.getParameter("cmd");
String message = "";
String root = request.getContextPath()+"/";

String forward = null;
DbControl dc = sc.newDbControl();
try
{
	if ("SaveSettings".equals(cmd))
	{
		String email = Values.getStringOrNull(request.getParameter("email"));
		if (email != null && !HTML.isValidEmail(email))
		{
			throw new WebException("popup", "Invalid email address",
				"The email address {1} is not a valid email address.",
				HTML.encodeTags(email)
			);
		}
		User user = sc.getSessionSetting("user");
		dc.reattachItem(user, false);

		JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, null, user);
		ExtensionsInvoker<OnSaveAction> invoker = ExtensionsControl.useExtensions(jspContext, EditUtil.EP_PREFIX_SAVE+"user-information");
		try
		{
			// Contact information tab
			user.setEmail(email);
			boolean useDeviceVerification = Values.getBoolean(request.getParameter("useDeviceVerification"));
			if (!useDeviceVerification) user.disableDeviceVerification();
			if (EmailUtil.isEnabled())
			{
				user.setSendMessagesAsEmail(email != null && Values.getBoolean(request.getParameter("sendMessagesAsEmail")));
				if (useDeviceVerification && !user.getUseDeviceVerification())
				{
					// Send a verification code for verifying the email address
					forward = "verify_email.jsp?ID="+ID;
				}
			}
			user.setOrganisation(Values.getStringOrNull(request.getParameter("organisation")));
			user.setAddress(Values.getStringOrNull(request.getParameter("address")));
			user.setPhone(Values.getStringOrNull(request.getParameter("phone")));
			user.setFax(Values.getStringOrNull(request.getParameter("fax")));
			user.setUrl(Values.getStringOrNull(request.getParameter("url")));
	
			// Password tab
			String password = Values.getStringOrNull(request.getParameter("password"));
			if (password != null) user.setPassword(password);
	
			// Other settings tab
			user.setDescription(Values.getStringOrNull(request.getParameter("description")));
	
			// Extended properties
			List<ExtendedProperty> extendedProperties = ExtendedProperties.getProperties("UserData", true);
			if (extendedProperties != null)
			{
				Map<String, String[]> parameters = request.getParameterMap();
				for (ExtendedProperty ep : extendedProperties)
				{
					if (ep.isRestrictedEdit()) continue;
					String name = ep.getName();
					if (parameters.containsKey("ep."+name))
					{
						Formatter<?> formatter = FormatterFactory.getExtendedPropertyFormatter(sc, ep);
						Object value = formatter.parseString(Values.getStringOrNull(request.getParameter("ep."+name)));
						user.setExtended(name, value);
					}
				}
			}
			
			// OnSave extensions
			invoker.render(OnSaveRenderer.ON_SAVE);
			dc.commit();
			invoker.render(OnSaveRenderer.ON_COMMIT);
			message = "Information saved";
		}
		catch (Exception ex)
		{
			invoker.render(OnSaveRenderer.onRollback(ex));
			throw ex;
		}
		finally
		{
			sc.setSessionSetting("user", null);
		}
	}
	else if ("VerifyEmail".equals(cmd))
	{
		String verificationCode = request.getParameter("verificationCode");
		User user = sc.getSessionSetting("user");
		dc.reattachItem(user, false);
		user.enableDeviceVerification(verificationCode);
		message = "The email has been verified";
		dc.commit();
	}
	else if ("SavePreferences".equals(cmd))
	{
		User user = User.getById(dc, sc.getLoggedInUserId());
		JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, null, user);
		ExtensionsInvoker<OnSaveAction> invoker = ExtensionsControl.useExtensions(jspContext, EditUtil.EP_PREFIX_SAVE+"user-information");
		try
		{
			int newScale = Values.getInt(request.getParameter("scale"), 100);
			sc.setUserClientSetting("appearance.scale", Integer.toString(newScale));
			sc.setUserClientSetting("appearance.fontsize", Values.getString(request.getParameter("fontsize"), "size_m.css"));
			sc.setSessionSetting("appearance.scale", new Float(newScale / 100.0));
			sc.setUserClientSetting("text.long", request.getParameter("longTexts"));
			ItemContext cc = sc.getCurrentContext(Item.USERCLIENTSETTING);
			int maxRecent = Base.getMaxRecent(sc);
		
			String toolbar = request.getParameter("toolbar");
			boolean hasImages = true;
			boolean hasText = true;
			if ("text".equals(toolbar))
			{
				hasImages = false;
			}
			else if ("images".equals(toolbar))
			{
				hasText = false;
			}
			sc.setUserClientSetting("toolbar.images", hasImages ? "1" : "0");
			sc.setUserClientSetting("toolbar.text", hasText ? "1" : "0");
			String minColor = Values.getString(request.getParameter("mincolor"), "0000FF");
			String midColor = Values.getString(request.getParameter("midcolor"), "FFFFFF");
			String maxColor = Values.getString(request.getParameter("maxcolor"), "FFFF00");
			sc.setUserClientSetting("ratiocolor.min", minColor);
			sc.setUserClientSetting("ratiocolor.mid", midColor);
			sc.setUserClientSetting("ratiocolor.max", maxColor);
			cc.setRecent("colors", minColor + "," + midColor + "," + maxColor, maxRecent);
			
			String dateFormat = request.getParameter("date_format");
			String dateTimeFormat = request.getParameter("datetime_format");
			FormatterSettings.setDateFormat(sc, dateFormat);
			FormatterSettings.setDateTimeFormat(sc, dateTimeFormat);
			cc.setRecent("dateFormats", dateFormat, maxRecent);
			cc.setRecent("dateTimeFormats", dateTimeFormat, maxRecent);
	
			int numDecimals = Values.getInt(request.getParameter("decimals"), 2);
			FormatterSettings.setNumDecimals(sc, numDecimals);
			
			sc.setUserClientSetting("dialogs.remember-positions", Values.getBoolean(request.getParameter("remember_positions")) ? "1" : "0");
			
			sc.setUserClientSetting("start-page-id", Values.getStringOrNull(request.getParameter("start_page")));
			
			// Plugins tab
			sc.setUserClientSetting("plugins.sendmessage", Values.getString(request.getParameter("sendmessage"), "0"));
			sc.setUserClientSetting("plugins.removejob", Values.getString(request.getParameter("removejob"), "0"));
			sc.setUserClientSetting("plugins.showWarnings", Values.getString(request.getParameter("show_warnings"), "0"));
			
			// Recent items tab
			int newMaxUsed = Values.getInt(request.getParameter("maxUsed"), 4);
			sc.setUserClientSetting("appearance.recent", Integer.toString(newMaxUsed));
			sc.setSessionSetting("appearance.recent", newMaxUsed);
			int newMaxViewed = Values.getInt(request.getParameter("maxViewed"), 6);
			sc.setUserClientSetting("menu.mostRecent.maxViewed", Integer.toString(newMaxViewed));
			sc.setUserClientSetting("menu.mostRecent.loadNames", Values.getString(request.getParameter("loadNames"), "0"));
	
			String[] stickyItems = request.getParameterValues("sticky_items");
			sc.setUserClientSetting("menu.mostRecent", 
				stickyItems == null ? "" : Values.getString(Arrays.asList(stickyItems), ":", true));
			
			// Roles tab
			String[] inactiveRoles = request.getParameterValues("inactiveRoles");
			sc.setUserClientSetting("inactiveRoles", inactiveRoles == null ? null : Values.getString(Arrays.asList(inactiveRoles), ":", true));
			
			// OnSave extensions
			invoker.render(OnSaveRenderer.ON_SAVE);
			dc.commit();
			invoker.render(OnSaveRenderer.ON_COMMIT);
			message = "Preferences saved";
		}
		catch (Exception ex)
		{
			invoker.render(OnSaveRenderer.onRollback(ex));
			throw ex;
		}
	}
	else if ("ReloadPermissions".equals(cmd))
	{
		sc.reloadPermissions();
		sc.reloadSettings(true, false);
		sc.setSessionSetting("menu.standard.html", null);
		message = "Permissions reloaded";
	}
	else if ("ResetFilters".equals(cmd))
	{
		boolean inDatabase = Values.getBoolean(request.getParameter("database"));
		String itemType = Values.getStringOrNull(request.getParameter("item_type"));
		Item item = itemType == null ? null : Item.valueOf(itemType);
		int numDeleted = sc.deleteCurrentContexts(item, true, inDatabase);
		message = numDeleted + " list settings deleted";
	}
	else
	{
		throw new WebException("popup", "Invalid command", "The command {1} is not recognised as a valid command.", cmd);
	}
}
finally
{
	if (dc != null) dc.close();
}

if (forward != null)
{
	pageContext.forward(forward);
}
else
{
	response.sendRedirect(root + "common/close_popup.jsp?ID="+ID+"&refresh_opener=1&&message="+HTML.urlEncode(message));
}
%>

